@@ -27,67 +27,67 @@
 #if defined(__linux__)
 
 #define _SYS_LINUX_
-#define ESTSYSNAME  "Linux"
+#define TCSYSNAME  "Linux"
 
 #elif defined(__FreeBSD__)
 
 #define _SYS_FREEBSD_
-#define ESTSYSNAME  "FreeBSD"
+#define TCSYSNAME  "FreeBSD"
 
 #elif defined(__NetBSD__)
 
 #define _SYS_NETBSD_
-#define ESTSYSNAME  "NetBSD"
+#define TCSYSNAME  "NetBSD"
 
 #elif defined(__OpenBSD__)
 
 #define _SYS_OPENBSD_
-#define ESTSYSNAME  "OpenBSD"
+#define TCSYSNAME  "OpenBSD"
 
 #elif defined(__sun__)
 
 #define _SYS_SUNOS_
-#define ESTSYSNAME  "SunOS"
+#define TCSYSNAME  "SunOS"
 
 #elif defined(__hpux)
 
 #define _SYS_HPUX_
-#define ESTSYSNAME  "HP-UX"
+#define TCSYSNAME  "HP-UX"
 
 #elif defined(__osf)
 
 #define _SYS_TRU64_
-#define ESTSYSNAME  "Tru64"
+#define TCSYSNAME  "Tru64"
 
 #elif defined(_AIX)
 
 #define _SYS_AIX_
-#define ESTSYSNAME  "AIX"
+#define TCSYSNAME  "AIX"
 
 #elif defined(__APPLE__) && defined(__MACH__)
 
 #define _SYS_MACOSX_
-#define ESTSYSNAME  "Mac OS X"
+#define TCSYSNAME  "Mac OS X"
 
 #elif defined(_MSC_VER)
 
 #define _SYS_MSVC_
-#define ESTSYSNAME  "Windows (VC++)"
+#define TCSYSNAME  "Windows (VC++)"
 
 #elif defined(_WIN32)
 
 #define _SYS_MINGW_
-#define ESTSYSNAME  "Windows (MinGW)"
+#define TCSYSNAME  "Windows (MinGW)"
 
 #elif defined(__CYGWIN__)
 
 #define _SYS_CYGWIN_
-#define ESTSYSNAME  "Windows (Cygwin)"
+#define TCSYSNAME  "Windows (Cygwin)"
 
 #else
 
 #define _SYS_GENERIC_
-#define ESTSYSNAME  "Generic"
+#define TCSYSNAME  "Generic"
 
 #endif
 
@@ -159,6 +159,24 @@
 #define TCUSEZLIB      1
 #endif
 
+#if defined(_MYNOBZIP)
+#define TCUSEBZIP      0
+#else
+#define TCUSEBZIP      1
+#endif
+
+#if defined(_MYEXLZMA)
+#define TCUSEEXLZMA    1
+#else
+#define TCUSEEXLZMA    0
+#endif
+
+#if defined(_MYEXLZO)
+#define TCUSEEXLZO     1
+#else
+#define TCUSEEXLZO     0
+#endif
+
 #if defined(_MYNOPTHREAD)
 #define TCUSEPTHREAD   0
 #else
@@ -171,10 +189,10 @@
 #define TCMICROYIELD   0
 #endif
 
-#define sizeof(a)      ((int)sizeof(a))
-
-int _tc_dummyfunc(void);
-int _tc_dummyfuncv(int a, ...);
+#define MYMALLOC       malloc
+#define MYCALLOC       calloc
+#define MYREALLOC      realloc
+#define MYFREE         free
 
 
 
@@ -211,6 +229,8 @@ int _tc_dummyfuncv(int a, ...);
 #include <sys/times.h>
 #include <fcntl.h>
 #include <dirent.h>
+#include <regex.h>
+#include <glob.h>
 
 #if TCUSEPTHREAD
 #include <pthread.h>
@@ -218,6 +238,18 @@ int _tc_dummyfuncv(int a, ...);
 
 
 
+/*************************************************************************************************
+ * miscellaneous hacks
+ *************************************************************************************************/
+
+
+#define sizeof(a)      ((int)sizeof(a))
+
+int _tc_dummyfunc(void);
+int _tc_dummyfuncv(int a, ...);
+
+
+
 /*************************************************************************************************
  * notation of filesystems
  *************************************************************************************************/
@@ -252,6 +284,28 @@ extern unsigned int (*_tc_getcrc)(const char *, int);
 
 
 
+/*************************************************************************************************
+ * for BZIP2
+ *************************************************************************************************/
+
+
+extern char *(*_tc_bzcompress)(const char *, int, int *);
+
+extern char *(*_tc_bzdecompress)(const char *, int, int *);
+
+
+
+/*************************************************************************************************
+ * for test of custom codec functions
+ *************************************************************************************************/
+
+
+void *_tc_recencode(const void *ptr, int size, int *sp, void *op);
+
+void *_tc_recdecode(const void *ptr, int size, int *sp, void *op);
+
+
+
 /*************************************************************************************************
  * for POSIX thread disability
  *************************************************************************************************/
@@ -259,39 +313,39 @@ extern unsigned int (*_tc_getcrc)(const char *, int);
 
 #if ! TCUSEPTHREAD
 
-#define pthread_once_t                   int
+#define pthread_once_t                   intptr_t
 #undef PTHREAD_ONCE_INIT
 #define PTHREAD_ONCE_INIT                0
-#define pthread_once(TC_a, TC_b)         _tc_dummyfuncv((int)(TC_a), (TC_b))
+#define pthread_once(TC_a, TC_b)         _tc_dummyfuncv((intptr_t)(TC_a), (TC_b))
 
-#define pthread_mutexattr_t              int
+#define pthread_mutexattr_t              intptr_t
 #undef PTHREAD_MUTEX_RECURSIVE
 #define PTHREAD_MUTEX_RECURSIVE          0
-#define pthread_mutexattr_init(TC_a)     _tc_dummyfuncv((int)(TC_a))
-#define pthread_mutexattr_destroy(TC_a)  _tc_dummyfuncv((int)(TC_a))
-#define pthread_mutexattr_settype(TC_a, TC_b)  _tc_dummyfuncv((int)(TC_a), (TC_b))
+#define pthread_mutexattr_init(TC_a)     _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_mutexattr_destroy(TC_a)  _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_mutexattr_settype(TC_a, TC_b)  _tc_dummyfuncv((intptr_t)(TC_a), (TC_b))
 
-#define pthread_mutex_t                  int
+#define pthread_mutex_t                  intptr_t
 #undef PTHREAD_MUTEX_INITIALIZER
 #define PTHREAD_MUTEX_INITIALIZER        0
-#define pthread_mutex_init(TC_a, TC_b)   _tc_dummyfuncv((int)(TC_a), (TC_b))
-#define pthread_mutex_destroy(TC_a)      _tc_dummyfuncv((int)(TC_a))
-#define pthread_mutex_lock(TC_a)         _tc_dummyfuncv((int)(TC_a))
-#define pthread_mutex_unlock(TC_a)       _tc_dummyfuncv((int)(TC_a))
+#define pthread_mutex_init(TC_a, TC_b)   _tc_dummyfuncv((intptr_t)(TC_a), (TC_b))
+#define pthread_mutex_destroy(TC_a)      _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_mutex_lock(TC_a)         _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_mutex_unlock(TC_a)       _tc_dummyfuncv((intptr_t)(TC_a))
 
-#define pthread_rwlock_t                 int
+#define pthread_rwlock_t                 intptr_t
 #undef PTHREAD_RWLOCK_INITIALIZER
 #define PTHREAD_RWLOCK_INITIALIZER       0
-#define pthread_rwlock_init(TC_a, TC_b)  _tc_dummyfuncv((int)(TC_a), (TC_b))
-#define pthread_rwlock_destroy(TC_a)     _tc_dummyfuncv((int)(TC_a))
-#define pthread_rwlock_rdlock(TC_a)      _tc_dummyfuncv((int)(TC_a))
-#define pthread_rwlock_wrlock(TC_a)      _tc_dummyfuncv((int)(TC_a))
-#define pthread_rwlock_unlock(TC_a)      _tc_dummyfuncv((int)(TC_a))
+#define pthread_rwlock_init(TC_a, TC_b)  _tc_dummyfuncv((intptr_t)(TC_a), (TC_b))
+#define pthread_rwlock_destroy(TC_a)     _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_rwlock_rdlock(TC_a)      _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_rwlock_wrlock(TC_a)      _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_rwlock_unlock(TC_a)      _tc_dummyfuncv((intptr_t)(TC_a))
 
-#define pthread_key_create(TC_a, TC_b)   _tc_dummyfuncv((int)(TC_a), (TC_b))
-#define pthread_key_delete(TC_a)         _tc_dummyfuncv((int)(TC_a))
-#define pthread_setspecific(TC_a, TC_b)  _tc_dummyfuncv((int)(TC_a))
-#define pthread_getspecific(TC_a)        _tc_dummyfuncv((int)(TC_a))
+#define pthread_key_create(TC_a, TC_b)   _tc_dummyfuncv((intptr_t)(TC_a), (TC_b))
+#define pthread_key_delete(TC_a)         _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_setspecific(TC_a, TC_b)  _tc_dummyfuncv((intptr_t)(TC_a))
+#define pthread_getspecific(TC_a)        _tc_dummyfuncv((intptr_t)(TC_a))
 
 #define pthread_create(TC_th, TC_attr, TC_func, TC_arg) \
   (*(TC_th) = 0, (TC_func)(TC_arg), 0)
