@@ -294,7 +294,7 @@ static int procread(const char *name){
       err = true;
       break;
     }
-    free(vbuf);
+    tcfree(vbuf);
     if(rnum > 250 && i % (rnum / 250) == 0){
       putchar('.');
       fflush(stdout);
@@ -365,7 +365,7 @@ static int procrcat(const char *name, int rnum){
   }
   for(int i = 1; i <= rnum; i++){
     char kbuf[RECBUFSIZ];
-    int ksiz = sprintf(kbuf, "%d", myrand(pnum));
+    int ksiz = sprintf(kbuf, "%d", myrand(pnum) + 1);
     if(!tcadbputcat(adb, kbuf, ksiz, kbuf, ksiz)){
       eprint(adb, "tcadbputcat");
       err = true;
@@ -428,10 +428,10 @@ static int procmisc(const char *name, int rnum){
     } else if(vsiz != ksiz || memcmp(vbuf, kbuf, vsiz)){
       eprint(adb, "(validation)");
       err = true;
-      free(vbuf);
+      tcfree(vbuf);
       break;
     }
-    free(vbuf);
+    tcfree(vbuf);
     if(rnum > 250 && i % (rnum / 250) == 0){
       putchar('.');
       fflush(stdout);
@@ -464,7 +464,7 @@ static int procmisc(const char *name, int rnum){
     if(rsiz != vsiz || memcmp(rbuf, vbuf, rsiz)){
       eprint(adb, "(validation)");
       err = true;
-      free(rbuf);
+      tcfree(rbuf);
       break;
     }
     if(rnum > 250 && i % (rnum / 250) == 0){
@@ -472,7 +472,7 @@ static int procmisc(const char *name, int rnum){
       fflush(stdout);
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
     }
-    free(rbuf);
+    tcfree(rbuf);
   }
   iprintf("word writing:\n");
   const char *words[] = {
@@ -567,11 +567,11 @@ static int procmisc(const char *name, int rnum){
     if(!vbuf){
       eprint(adb, "tcadbget");
       err = true;
-      free(kbuf);
+      tcfree(kbuf);
       break;
     }
-    free(vbuf);
-    free(kbuf);
+    tcfree(vbuf);
+    tcfree(kbuf);
     if(rnum > 250 && i % (rnum / 250) == 0){
       putchar('.');
       fflush(stdout);
@@ -617,7 +617,7 @@ static int procwicked(const char *name, int rnum){
   TCMAP *map = tcmapnew2(rnum / 5);
   for(int i = 1; i <= rnum && !err; i++){
     char kbuf[RECBUFSIZ];
-    int ksiz = sprintf(kbuf, "%d", myrand(rnum));
+    int ksiz = sprintf(kbuf, "%d", myrand(rnum) + 1);
     char vbuf[RECBUFSIZ];
     int vsiz = myrand(RECBUFSIZ);
     memset(vbuf, '*', vsiz);
@@ -682,11 +682,11 @@ static int procwicked(const char *name, int rnum){
       break;
     case 8:
       putchar('8');
-      if((rbuf = tcadbget(adb, kbuf, ksiz, &vsiz)) != NULL) free(rbuf);
+      if((rbuf = tcadbget(adb, kbuf, ksiz, &vsiz)) != NULL) tcfree(rbuf);
       break;
     case 9:
       putchar('9');
-      if((rbuf = tcadbget2(adb, kbuf)) != NULL) free(rbuf);
+      if((rbuf = tcadbget2(adb, kbuf)) != NULL) tcfree(rbuf);
       break;
     case 10:
       putchar('A');
@@ -707,7 +707,7 @@ static int procwicked(const char *name, int rnum){
       for(int j = myrand(rnum) / 1000 + 1; j >= 0; j--){
         int iksiz;
         char *ikbuf = tcadbiternext(adb, &iksiz);
-        if(ikbuf) free(ikbuf);
+        if(ikbuf) tcfree(ikbuf);
       }
       break;
     default:
@@ -746,7 +746,7 @@ static int procwicked(const char *name, int rnum){
         err = true;
       }
     }
-    free(rbuf);
+    tcfree(rbuf);
     if(i % 50 == 0) iprintf(" (%08d)\n", i);
   }
   if(rnum % 50 > 0) iprintf(" (%08d)\n", rnum);
@@ -766,7 +766,7 @@ static int procwicked(const char *name, int rnum){
       eprint(adb, "(validation)");
       err = true;
     }
-    free(rbuf);
+    tcfree(rbuf);
     if(!tcadbout(adb, kbuf, ksiz)){
       eprint(adb, "tcadbout");
       err = true;
