@@ -51,6 +51,16 @@ __TCBDB_CLINKAGEBEGIN
    are equivalent. */
 typedef int (*BDBCMP)(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
 
+/* type of the pointer to a encoding or decoding function.
+   `ptr' specifies the pointer to the region.
+   `size' specifies the size of the region.
+   `sp' specifies the pointer to the variable into which the size of the region of the return
+   value is assigned.
+   `op' specifies the pointer to the optional opaque object.
+   If successful, the return value is the pointer to the result object allocated with `malloc'
+   call, else, it is `NULL'. */
+typedef void *(*BDBCODEC)(const void *ptr, int size, int *sp, void *op);
+
 typedef struct {                         /* type of structure for a B+ tree database */
   void *mmtx;                            /* mutex for method */
   void *cmtx;                            /* mutex for cache */
@@ -75,6 +85,8 @@ typedef struct {                         /* type of structure for a B+ tree data
   uint32_t lcnum;                        /* max number of cached leaves */
   uint32_t ncnum;                        /* max number of cached nodes */
   uint32_t lsmax;                        /* max size of each leaf */
+  uint32_t lschk;                        /* counter for leaf size checking */
+  uint64_t capnum;                       /* capacity number of records */
   uint64_t *hist;                        /* history array of visited nodes */
   int hnum;                              /* number of element of the history array */
   uint64_t hleaf;                        /* ID number of the leaf referred by the history */
@@ -98,7 +110,9 @@ enum {                                   /* enumeration for additional flags */
 enum {                                   /* enumeration for tuning options */
   BDBTLARGE = 1 << 0,                    /* use 64-bit bucket array */
   BDBTDEFLATE = 1 << 1,                  /* compress each page with Deflate */
-  BDBTTCBS = 1 << 2                      /* compress each page with TCBS */
+  BDBTBZIP = 1 << 2,                     /* compress each record with BZIP2 */
+  BDBTTCBS = 1 << 3,                     /* compress each page with TCBS */
+  BDBTEXCODEC = 1 << 4                   /* compress each record with outer functions */
 };
 
 enum {                                   /* enumeration for open modes */
@@ -194,8 +208,8 @@ bool tcbdbsetcmpfunc(TCBDB *bdb, BDBCMP cmp, void *cmpop);
    is negative, the default value is specified.  The default value is 10 standing for 2^10=1024.
    `opts' specifies options by bitwise or: `BDBTLARGE' specifies that the size of the database
    can be larger than 2GB by using 64-bit bucket array, `BDBTDEFLATE' specifies that each page
-   is compressed with Deflate encoding, `BDBTTCBS' specifies that each page is compressed with
-   TCBS encoding.
+   is compressed with Deflate encoding, `BDBTBZIP' specifies that each page is compressed with
+   BZIP2 encoding, `BDBTTCBS' specifies that each page is compressed with TCBS encoding.
    If successful, the return value is true, else, it is false.
    Note that the tuning parameters should be set before the database is opened. */
 bool tcbdbtune(TCBDB *bdb, int32_t lmemb, int32_t nmemb,
@@ -507,7 +521,7 @@ TCLIST *tcbdbfwmkeys2(TCBDB *bdb, const char *pstr, int max);
 /* Synchronize updated contents of a B+ tree database object with the file and the device.
    `bdb' specifies the B+ tree database object connected as a writer.
    If successful, the return value is true, else, it is false.
-   This function is useful when another process connects the same database file. */
+   This function is useful when another process connects to the same database file. */
 bool tcbdbsync(TCBDB *bdb);
 
 
@@ -525,8 +539,9 @@ bool tcbdbsync(TCBDB *bdb);
    is negative, the current setting is not changed.
    `opts' specifies options by bitwise or: `BDBTLARGE' specifies that the size of the database
    can be larger than 2GB by using 64-bit bucket array, `BDBTDEFLATE' specifies that each record
-   is compressed with Deflate encoding, `BDBTTCBS' specifies that each page is compressed with
-   TCBS encoding.  If it is `UINT8_MAX', the current setting is not changed.
+   is compressed with Deflate encoding, `BDBTBZIP' specifies that each page is compressed with
+   BZIP2 encoding, `BDBTTCBS' specifies that each page is compressed with TCBS encoding.  If it
+   is `UINT8_MAX', the current setting is not changed.
    If successful, the return value is true, else, it is false.
    This function is useful to reduce the size of the database file with data fragmentation by
    successive updating. */
@@ -557,8 +572,10 @@ bool tcbdbcopy(TCBDB *bdb, const char *path);
    If successful, the return value is true, else, it is false.
    The database is locked by the thread while the transaction so that only one transaction can be
    activated with a database object at the same time.  Thus, the serializable isolation level is
-   assumed if every database operation is performed in the transaction.  If the database is
-   closed during transaction, the transaction is aborted implicitly. */
+   assumed if every database operation is performed in the transaction.  Because all pages are
+   cached on memory while the transaction, the amount of referred records is limited by the
+   memory capacity.  If the database is closed during transaction, the transaction is aborted
+   implicitly. */
 bool tcbdbtranbegin(TCBDB *bdb);
 
 
@@ -802,13 +819,31 @@ void tcbdbsetdbgfd(TCBDB *bdb, int fd);
 int tcbdbdbgfd(TCBDB *bdb);
 
 
-/* Synchronize updating contents on memory.
+/* Synchronize updating contents on memory of a B+ tree database object.
    `bdb' specifies the B+ tree database object connected as a writer.
    `phys' specifies whether to synchronize physically.
    If successful, the return value is true, else, it is false. */
 bool tcbdbmemsync(TCBDB *bdb, bool phys);
 
 
+/* Clear the cache of a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   If successful, the return value is true, else, it is false. */
+bool tcbdbcacheclear(TCBDB *bdb);
+
+
+/* Get the comparison function of a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   The return value is the pointer to the comparison function. */
+BDBCMP tcbdbcmpfunc(TCBDB *bdb);
+
+
+/* Get the opaque object for the comparison function of a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   The return value is the opaque object for the comparison function. */
+void *tcbdbcmpop(TCBDB *bdb);
+
+
 /* Get the maximum number of cached leaf nodes of a B+ tree database object.
    `bdb' specifies the B+ tree database object.
    The return value is the maximum number of cached leaf nodes. */
@@ -904,6 +939,30 @@ uint64_t tcbdbbnumused(TCBDB *bdb);
 bool tcbdbsetlsmax(TCBDB *bdb, uint32_t lsmax);
 
 
+/* Set the capacity number of records.
+   `bdb' specifies the B+ tree database object which is not opened.
+   `capnum' specifies the capacity number of records.  If it is not more than 0, the capacity is
+   unlimited.
+   If successful, the return value is true, else, it is false.
+   When the number of records exceeds the capacity, forehand records are removed implicitly.
+   Note that the tuning parameters of the database should be set before the database is opened. */
+bool tcbdbsetcapnum(TCBDB *bdb, uint64_t capnum);
+
+
+/* Set the custom codec functions of a B+ tree database object.
+   `bdb' specifies the B+ tree database object.
+   `enc' specifies the pointer to the custom encoding function.
+   `encop' specifies an arbitrary pointer to be given as a parameter of the encoding function.
+   If it is not needed, `NULL' can be specified.
+   `dec' specifies the pointer to the custom decoding function.
+   `decop' specifies an arbitrary pointer to be given as a parameter of the decoding function.
+   If it is not needed, `NULL' can be specified.
+   If successful, the return value is true, else, it is false.
+   Note that the custom codec functions should be set before the database is opened and should be
+   set every time the database is being opened. */
+bool tcbdbsetcodecfunc(TCBDB *bdb, BDBCODEC enc, void *encop, BDBCODEC dec, void *decop);
+
+
 /* Store a new record into a B+ tree database object with backward duplication.
    `bdb' specifies the B+ tree database object connected as a writer.
    `kbuf' specifies the pointer to the region of the key.
