@@ -31,6 +31,7 @@ static void printerr(TCBDB *bdb);
 static int printdata(const char *ptr, int size, bool px);
 static char *hextoobj(const char *str, int *sp);
 static char *mygetline(FILE *ifp);
+static int mycmpfunc(const char *aptr, int asiz, const char *bptr, int bsiz, void *op);
 static int runcreate(int argc, char **argv);
 static int runinform(int argc, char **argv);
 static int runput(int argc, char **argv);
@@ -94,7 +95,7 @@ static void usage(void){
   fprintf(stderr, "%s: the command line utility of the B+ tree database API\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s create [-cd|-ci|-cj] [-tl] [-td|-tb] path"
+  fprintf(stderr, "  %s create [-cd|-ci|-cj] [-tl] [-td|-tb|-tt|-tx] path"
           " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "  %s inform [-nl|-nb] path\n", g_progname);
   fprintf(stderr, "  %s put [-cd|-ci|-cj] [-nl|-nb] [-sx] [-dk|-dc|-dd|-db] path"
@@ -103,7 +104,7 @@ static void usage(void){
   fprintf(stderr, "  %s get [-cd|-ci|-cj] [-nl|-nb] [-sx] [-px] [-pz] path key\n", g_progname);
   fprintf(stderr, "  %s list [-cd|-ci|-cj] [-nl|-nb] [-m num] [-bk] [-pv] [-px] [-j str]"
           " [-rb bkey ekey] [-fm str] path\n", g_progname);
-  fprintf(stderr, "  %s optimize [-cd|-ci|-cj] [-tl] [-td|-tb] [-tz] [-nl|-nb] path"
+  fprintf(stderr, "  %s optimize [-cd|-ci|-cj] [-tl] [-td|-tb|-tt|-tx] [-tz] [-nl|-nb] path"
           " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "  %s importtsv [-nl|-nb] [-sc] path [file]\n", g_progname);
   fprintf(stderr, "  %s version\n", g_progname);
@@ -160,24 +161,37 @@ static char *hextoobj(const char *str, int *sp){
 
 /* read a line from a file descriptor */
 static char *mygetline(FILE *ifp){
-  char *buf;
-  int c, len, blen;
-  buf = NULL;
-  len = 0;
-  blen = 256;
+  int len = 0;
+  int blen = 1024;
+  char *buf = tcmalloc(blen);
+  bool end = true;
+  int c;
   while((c = fgetc(ifp)) != EOF){
-    if(blen <= len) blen *= 2;
-    buf = tcrealloc(buf, blen + 1);
+    end = false;
+    if(c == '\0') continue;
+    if(blen <= len){
+      blen *= 2;
+      buf = tcrealloc(buf, blen + 1);
+    }
     if(c == '\n' || c == '\r') c = '\0';
     buf[len++] = c;
     if(c == '\0') break;
   }
-  if(!buf) return NULL;
+  if(end){
+    tcfree(buf);
+    return NULL;
+  }
   buf[len] = '\0';
   return buf;
 }
 
 
+/* dummy comparison function */
+static int mycmpfunc(const char *aptr, int asiz, const char *bptr, int bsiz, void *op){
+  return 0;
+}
+
+
 /* parse arguments of create command */
 static int runcreate(int argc, char **argv){
   char *path = NULL;
@@ -201,7 +215,11 @@ static int runcreate(int argc, char **argv){
       } else if(!strcmp(argv[i], "-td")){
         opts |= BDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        opts |= BDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         opts |= BDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= BDBTEXCODEC;
       } else {
         usage();
       }
@@ -314,8 +332,8 @@ static int runput(int argc, char **argv){
     vbuf = tcmemdup(value, vsiz);
   }
   int rv = procput(path, kbuf, ksiz, vbuf, vsiz, cmp, omode, dmode);
-  free(vbuf);
-  free(kbuf);
+  tcfree(vbuf);
+  tcfree(kbuf);
   return rv;
 }
 
@@ -362,7 +380,7 @@ static int runout(int argc, char **argv){
     kbuf = tcmemdup(key, ksiz);
   }
   int rv = procout(path, kbuf, ksiz, cmp, omode);
-  free(kbuf);
+  tcfree(kbuf);
   return rv;
 }
 
@@ -415,7 +433,7 @@ static int runget(int argc, char **argv){
     kbuf = tcmemdup(key, ksiz);
   }
   int rv = procget(path, kbuf, ksiz, cmp, omode, px, pz);
-  free(kbuf);
+  tcfree(kbuf);
   return rv;
 }
 
@@ -506,8 +524,14 @@ static int runoptimize(int argc, char **argv){
         if(opts == UINT8_MAX) opts = 0;
         opts |= BDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        if(opts == UINT8_MAX) opts = 0;
+        opts |= BDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         if(opts == UINT8_MAX) opts = 0;
         opts |= BDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        if(opts == UINT8_MAX) opts = 0;
+        opts |= BDBTEXCODEC;
       } else if(!strcmp(argv[i], "-tz")){
         if(opts == UINT8_MAX) opts = 0;
       } else if(!strcmp(argv[i], "-nl")){
@@ -588,6 +612,7 @@ static int proccreate(const char *path, int lmemb, int nmemb,
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(bdb);
   if(!tcbdbtune(bdb, lmemb, nmemb, bnum, apow, fpow, opts)){
     printerr(bdb);
     tcbdbdel(bdb);
@@ -612,6 +637,8 @@ static int proccreate(const char *path, int lmemb, int nmemb,
 static int procinform(const char *path, int omode){
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
+  tcbdbsetcmpfunc(bdb, mycmpfunc, NULL);
+  tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL);
   if(!tcbdbopen(bdb, path, BDBOREADER | omode)){
     printerr(bdb);
     tcbdbdel(bdb);
@@ -627,6 +654,20 @@ static int procinform(const char *path, int omode){
   if(flags & BDBFOPEN) printf(" open");
   if(flags & BDBFFATAL) printf(" fatal");
   printf("\n");
+  BDBCMP cmp = tcbdbcmpfunc(bdb);
+  printf("comparison function: ");
+  if(cmp == tcbdbcmplexical){
+    printf("lexical");
+  } else if(cmp == tcbdbcmpdecimal){
+    printf("decimal");
+  } else if(cmp == tcbdbcmpint32){
+    printf("int32");
+  } else if(cmp == tcbdbcmpint64){
+    printf("int64");
+  } else {
+    printf("custom");
+  }
+  printf("\n");
   printf("max leaf member: %d\n", tcbdblmemb(bdb));
   printf("max node member: %d\n", tcbdbnmemb(bdb));
   printf("leaf number: %llu\n", (unsigned long long)tcbdblnum(bdb));
@@ -644,7 +685,9 @@ static int procinform(const char *path, int omode){
   printf("options:");
   if(opts & BDBTLARGE) printf(" large");
   if(opts & BDBTDEFLATE) printf(" deflate");
+  if(opts & BDBTBZIP) printf(" bzip");
   if(opts & BDBTTCBS) printf(" tcbs");
+  if(opts & BDBTEXCODEC) printf(" excodec");
   printf("\n");
   printf("record number: %llu\n", (unsigned long long)tcbdbrnum(bdb));
   printf("file size: %llu\n", (unsigned long long)tcbdbfsiz(bdb));
@@ -663,6 +706,7 @@ static int procput(const char *path, const char *kbuf, int ksiz, const char *vbu
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(bdb);
   if(!tcbdbopen(bdb, path, BDBOWRITER | omode)){
     printerr(bdb);
     tcbdbdel(bdb);
@@ -715,6 +759,7 @@ static int procout(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(bdb);
   if(!tcbdbopen(bdb, path, BDBOWRITER | omode)){
     printerr(bdb);
     tcbdbdel(bdb);
@@ -740,6 +785,7 @@ static int procget(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(bdb);
   if(!tcbdbopen(bdb, path, BDBOREADER | omode)){
     printerr(bdb);
     tcbdbdel(bdb);
@@ -751,7 +797,7 @@ static int procget(const char *path, const char *kbuf, int ksiz, BDBCMP cmp, int
   if(vbuf){
     printdata(vbuf, vsiz, px);
     if(!pz) putchar('\n');
-    free(vbuf);
+    tcfree(vbuf);
   } else {
     printerr(bdb);
     err = true;
@@ -771,6 +817,7 @@ static int proclist(const char *path, BDBCMP cmp, int omode, int max, bool pv, b
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(bdb);
   if(!tcbdbopen(bdb, path, BDBOREADER | omode)){
     printerr(bdb);
     tcbdbdel(bdb);
@@ -878,6 +925,7 @@ static int procoptimize(const char *path, int lmemb, int nmemb,
   TCBDB *bdb = tcbdbnew();
   if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   if(cmp && !tcbdbsetcmpfunc(bdb, cmp, NULL)) printerr(bdb);
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(bdb);
   if(!tcbdbopen(bdb, path, BDBOWRITER | omode)){
     printerr(bdb);
     tcbdbdel(bdb);
@@ -899,17 +947,18 @@ static int procoptimize(const char *path, int lmemb, int nmemb,
 
 /* perform importtsv command */
 static int procimporttsv(const char *path, const char *file, int omode, bool sc){
-  TCBDB *bdb = tcbdbnew();
-  if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
   FILE *ifp = file ? fopen(file, "rb") : stdin;
   if(!ifp){
     fprintf(stderr, "%s: could not open\n", file ? file : "(stdin)");
-    tcbdbdel(bdb);
     return 1;
   }
+  TCBDB *bdb = tcbdbnew();
+  if(g_dbgfd >= 0) tcbdbsetdbgfd(bdb, g_dbgfd);
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(bdb);
   if(!tcbdbopen(bdb, path, BDBOWRITER | BDBOCREAT | omode)){
     printerr(bdb);
     tcbdbdel(bdb);
+    if(ifp != stdin) fclose(ifp);
     return 1;
   }
   bool err = false;
@@ -917,14 +966,17 @@ static int procimporttsv(const char *path, const char *file, int omode, bool sc)
   int cnt = 0;
   while(!err && (line = mygetline(ifp)) != NULL){
     char *pv = strchr(line, '\t');
-    if(!pv) continue;
+    if(!pv){
+      tcfree(line);
+      continue;
+    }
     *pv = '\0';
     if(sc) tcstrtolower(line);
     if(!tcbdbputdup2(bdb, line, pv + 1) && tcbdbecode(bdb) != TCEKEEP){
       printerr(bdb);
       err = true;
     }
-    free(line);
+    tcfree(line);
     if(cnt > 0 && cnt % 100 == 0){
       putchar('.');
       fflush(stdout);
