@@ -122,13 +122,13 @@ static void usage(void){
   fprintf(stderr, "%s: test cases of the B+ tree database API of Tokyo Cabinet\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s write [-tl] [-td|-tb] [-nl|-nb] [-rnd] path tnum rnum"
+  fprintf(stderr, "  %s write [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-rnd] path tnum rnum"
           " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "  %s read [-nl|-nb] [-wb] [-rnd] path tnum\n", g_progname);
   fprintf(stderr, "  %s remove [-nl|-nb] [-rnd] path tnum\n", g_progname);
-  fprintf(stderr, "  %s wicked [-tl] [-td|-tb] [-nl|-nb] [-nc] path tnum rnum\n",
+  fprintf(stderr, "  %s wicked [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-nc] path tnum rnum\n",
           g_progname);
-  fprintf(stderr, "  %s typical [-tl] [-td|-tb] [-nl|-nb] [-nc] [-rr] path tnum rnum"
+  fprintf(stderr, "  %s typical [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-nc] [-rr] path tnum rnum"
           " [lmemb [nmemb [bnum [apow [fpow]]]]]\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
@@ -220,7 +220,11 @@ static int runwrite(int argc, char **argv){
       } else if(!strcmp(argv[i], "-td")){
         opts |= BDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        opts |= BDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         opts |= BDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= BDBTEXCODEC;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -348,7 +352,11 @@ static int runwicked(int argc, char **argv){
       } else if(!strcmp(argv[i], "-td")){
         opts |= BDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        opts |= BDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         opts |= BDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= BDBTEXCODEC;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -398,7 +406,11 @@ static int runtypical(int argc, char **argv){
       } else if(!strcmp(argv[i], "-td")){
         opts |= BDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        opts |= BDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         opts |= BDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= BDBTEXCODEC;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= BDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -459,6 +471,10 @@ static int procwrite(const char *path, int tnum, int rnum, int lmemb, int nmemb,
     eprint(bdb, "tcbdbsetmutex");
     err = true;
   }
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(bdb, "tcbdbsetcodecfunc");
+    err = true;
+  }
   if(!tcbdbtune(bdb, lmemb, nmemb, bnum, apow, fpow, opts)){
     eprint(bdb, "tcbdbtune");
     err = true;
@@ -524,6 +540,10 @@ static int procread(const char *path, int tnum, int omode, bool wb, bool rnd){
     eprint(bdb, "tcbdbsetmutex");
     err = true;
   }
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(bdb, "tcbdbsetcodecfunc");
+    err = true;
+  }
   if(!tcbdbopen(bdb, path, BDBOREADER | omode)){
     eprint(bdb, "tcbdbopen");
     err = true;
@@ -587,6 +607,10 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
     eprint(bdb, "tcbdbsetmutex");
     err = true;
   }
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(bdb, "tcbdbsetcodecfunc");
+    err = true;
+  }
   if(!tcbdbopen(bdb, path, BDBOWRITER | omode)){
     eprint(bdb, "tcbdbopen");
     err = true;
@@ -649,6 +673,10 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
     eprint(bdb, "tcbdbsetmutex");
     err = true;
   }
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(bdb, "tcbdbsetcodecfunc");
+    err = true;
+  }
   if(!tcbdbtune(bdb, 10, 10, rnum / 50, 10, -1, opts)){
     eprint(bdb, "tcbdbtune");
     err = true;
@@ -674,7 +702,6 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
       targs[i].nc = nc;
       targs[i].id = i;
       targs[i].map = map;
-      targs[i].map = map;
       if(pthread_create(threads + i, NULL, threadwicked, targs + i) != 0){
         eprint(bdb, "pthread_create");
         targs[i].id = -1;
@@ -725,7 +752,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
           err = true;
         }
       }
-      free(rbuf);
+      tcfree(rbuf);
       if(i % 50 == 0) iprintf(" (%08d)\n", i);
     }
     if(rnum % 50 > 0) iprintf(" (%08d)\n", rnum);
@@ -759,6 +786,10 @@ static int proctypical(const char *path, int tnum, int rnum, int lmemb, int nmem
     eprint(bdb, "tcbdbsetmutex");
     err = true;
   }
+  if(!tcbdbsetcodecfunc(bdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(bdb, "tcbdbsetcodecfunc");
+    err = true;
+  }
   if(!tcbdbtune(bdb, lmemb, nmemb, bnum, apow, fpow, opts)){
     eprint(bdb, "tcbdbtune");
     err = true;
@@ -866,7 +897,7 @@ static void *threadread(void *targ){
         eprint(bdb, "tcbdbget");
         err = true;
       }
-      free(vbuf);
+      tcfree(vbuf);
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
       putchar('.');
@@ -1030,7 +1061,7 @@ static void *threadwicked(void *targ){
         err = true;
       }
       if(!nc) tcmapput(map, kbuf, ksiz, rbuf, vsiz);
-      free(rbuf);
+      tcfree(rbuf);
       break;
     case 11:
       if(id == 0) putchar('B');
@@ -1038,20 +1069,20 @@ static void *threadwicked(void *targ){
         eprint(bdb, "tcbdbget");
         err = true;
       }
-      free(rbuf);
+      tcfree(rbuf);
       break;
     case 12:
       if(id == 0) putchar('C');
       if(!(rbuf = tcbdbget2(bdb, kbuf)) && tcbdbecode(bdb) != TCENOREC){
-        eprint(bdb, "tcbdbget");
+        eprint(bdb, "tcbdbget2");
         err = true;
       }
-      free(rbuf);
+      tcfree(rbuf);
       break;
     case 13:
       if(id == 0) putchar('D');
       if(!tcbdbget3(bdb, kbuf, ksiz, &vsiz) && tcbdbecode(bdb) != TCENOREC){
-        eprint(bdb, "tcbdbget");
+        eprint(bdb, "tcbdbget3");
         err = true;
       }
       break;
@@ -1200,12 +1231,12 @@ static void *threadtypical(void *targ){
             int vsiz;
             char *vbuf = tcbdbcurval(cur, &vsiz);
             if(vbuf){
-              free(vbuf);
+              tcfree(vbuf);
             } else if(tcbdbecode(bdb) != TCENOREC){
               eprint(bdb, "tcbdbcurval");
               err = true;
             }
-            free(kbuf);
+            tcfree(kbuf);
           } else if(tcbdbecode(bdb) != TCENOREC){
             eprint(bdb, "tcbdbcurkey");
             err = true;
@@ -1225,12 +1256,12 @@ static void *threadtypical(void *targ){
             int vsiz;
             char *vbuf = tcbdbcurval(cur, &vsiz);
             if(vbuf){
-              free(vbuf);
+              tcfree(vbuf);
             } else if(tcbdbecode(bdb) != TCENOREC){
               eprint(bdb, "tcbdbcurval");
               err = true;
             }
-            free(kbuf);
+            tcfree(kbuf);
           } else if(tcbdbecode(bdb) != TCENOREC){
             eprint(bdb, "tcbdbcurkey");
             err = true;
@@ -1250,12 +1281,12 @@ static void *threadtypical(void *targ){
             int vsiz;
             char *vbuf = tcbdbcurval(cur, &vsiz);
             if(vbuf){
-              free(vbuf);
+              tcfree(vbuf);
             } else if(tcbdbecode(bdb) != TCENOREC){
               eprint(bdb, "tcbdbcurval");
               err = true;
             }
-            free(kbuf);
+            tcfree(kbuf);
           } else if(tcbdbecode(bdb) != TCENOREC){
             eprint(bdb, "tcbdbcurkey");
             err = true;
@@ -1271,12 +1302,12 @@ static void *threadtypical(void *targ){
         if(map){
           int msiz;
           const char *mbuf = tcmapget(map, buf, len, &msiz);
-          if(msiz != vsiz || memcmp(mbuf, vbuf, vsiz)){
+          if(!mbuf || msiz != vsiz || memcmp(mbuf, vbuf, vsiz)){
             eprint(bdb, "(validation)");
             err = true;
           }
         }
-        free(vbuf);
+        tcfree(vbuf);
       } else {
         if(tcbdbecode(bdb) != TCENOREC){
           eprint(bdb, "tcbdbget");
@@ -1309,7 +1340,7 @@ static void *threadtypical(void *targ){
           eprint(bdb, "(validation)");
           err = true;
         }
-        free(vbuf);
+        tcfree(vbuf);
       } else {
         eprint(bdb, "(validation)");
         err = true;
