@@ -40,8 +40,20 @@ __TCHDB_CLINKAGEBEGIN
  *************************************************************************************************/
 
 
+/* type of the pointer to a encoding or decoding function.
+   `ptr' specifies the pointer to the region.
+   `size' specifies the size of the region.
+   `sp' specifies the pointer to the variable into which the size of the region of the return
+   value is assigned.
+   `op' specifies the pointer to the optional opaque object.
+   If successful, the return value is the pointer to the result object allocated with `malloc'
+   call, else, it is `NULL'. */
+typedef void *(*HDBCODEC)(const void *ptr, int size, int *sp, void *op);
+
 typedef struct {                         /* type of structure for a hash database */
   void *mmtx;                            /* mutex for method */
+  void *rmtxs;                           /* mutexes for records */
+  void *amtx;                            /* mutex for attributes */
   void *dmtx;                            /* mutex for DRP */
   void *eckey;                           /* key for thread specific error code */
   uint8_t type;                          /* database type */
@@ -76,6 +88,10 @@ typedef struct {                         /* type of structure for a hash databas
   uint64_t drpoff;                       /* offset of the delayed record pool */
   TCMDB *recc;                           /* cache for records */
   uint32_t rcnum;                        /* max number of cached records */
+  HDBCODEC enc;                          /* pointer to the encoding function */
+  void *encop;                           /* opaque object for the encoding functions */
+  HDBCODEC dec;                          /* pointer to the decoding function */
+  void *decop;                           /* opaque object for the decoding functions */
   int ecode;                             /* last happened error code */
   bool fatal;                            /* whether a fatal error occured */
   uint64_t inode;                        /* inode number */
@@ -97,38 +113,6 @@ typedef struct {                         /* type of structure for a hash databas
   int64_t cnt_adjrecc;                   /* tesing counter for record cache adjust times */
 } TCHDB;
 
-enum {                                   /* enumeration for error codes */
-  TCESUCCESS,                            /* success */
-  TCETHREAD,                             /* threading error */
-  TCEINVALID,                            /* invalid operation */
-  TCENOFILE,                             /* file not found */
-  TCENOPERM,                             /* no permission */
-  TCEMETA,                               /* invalid meta data */
-  TCERHEAD,                              /* invalid record header */
-  TCEOPEN,                               /* open error */
-  TCECLOSE,                              /* close error */
-  TCETRUNC,                              /* trunc error */
-  TCESYNC,                               /* sync error */
-  TCESTAT,                               /* stat error */
-  TCESEEK,                               /* seek error */
-  TCEREAD,                               /* read error */
-  TCEWRITE,                              /* write error */
-  TCEMMAP,                               /* mmap error */
-  TCELOCK,                               /* lock error */
-  TCEUNLINK,                             /* unlink error */
-  TCERENAME,                             /* rename error */
-  TCEMKDIR,                              /* mkdir error */
-  TCERMDIR,                              /* rmdir error */
-  TCEKEEP,                               /* existing record */
-  TCENOREC,                              /* no record found */
-  TCEMISC = 9999                         /* miscellaneous error */
-};
-
-enum {                                   /* enumeration for database type */
-  HDBTHASH,                              /* hash table */
-  HDBTBTREE                              /* B+ tree */
-};
-
 enum {                                   /* enumeration for additional flags */
   HDBFOPEN = 1 << 0,                     /* whether opened */
   HDBFFATAL = 1 << 1                     /* whetehr with fatal error */
@@ -137,7 +121,9 @@ enum {                                   /* enumeration for additional flags */
 enum {                                   /* enumeration for tuning options */
   HDBTLARGE = 1 << 0,                    /* use 64-bit bucket array */
   HDBTDEFLATE = 1 << 1,                  /* compress each record with Deflate */
-  HDBTTCBS = 1 << 2                      /* compress each record with TCBS */
+  HDBTBZIP = 1 << 2,                     /* compress each record with BZIP2 */
+  HDBTTCBS = 1 << 3,                     /* compress each record with TCBS */
+  HDBTEXCODEC = 1 << 4                   /* compress each record with custom functions */
 };
 
 enum {                                   /* enumeration for open modes */
@@ -202,8 +188,8 @@ bool tchdbsetmutex(TCHDB *hdb);
    is negative, the default value is specified.  The default value is 10 standing for 2^10=1024.
    `opts' specifies options by bitwise or: `HDBTLARGE' specifies that the size of the database
    can be larger than 2GB by using 64-bit bucket array, `HDBTDEFLATE' specifies that each record
-   is compressed with Deflate encoding, `HDBTTCBS' specifies that each record is compressed with
-   TCBS encoding.
+   is compressed with Deflate encoding, `HDBTBZIP' specifies that each record is compressed with
+   BZIP2 encoding, `HDBTTCBS' specifies that each record is compressed with TCBS encoding.
    If successful, the return value is true, else, it is false.
    Note that the tuning parameters should be set before the database is opened. */
 bool tchdbtune(TCHDB *hdb, int64_t bnum, int8_t apow, int8_t fpow, uint8_t opts);
@@ -468,7 +454,7 @@ TCLIST *tchdbfwmkeys2(TCHDB *hdb, const char *pstr, int max);
 /* Synchronize updated contents of a hash database object with the file and the device.
    `hdb' specifies the hash database object connected as a writer.
    If successful, the return value is true, else, it is false.
-   This function is useful when another process connects the same database file. */
+   This function is useful when another process connects to the same database file. */
 bool tchdbsync(TCHDB *hdb);
 
 
@@ -482,8 +468,9 @@ bool tchdbsync(TCHDB *hdb);
    is negative, the current setting is not changed.
    `opts' specifies options by bitwise or: `HDBTLARGE' specifies that the size of the database
    can be larger than 2GB by using 64-bit bucket array, `HDBTDEFLATE' specifies that each record
-   is compressed with Deflate encoding, `HDBTTCBS' specifies that each record is compressed with
-   TCBS encoding.  If it is `UINT8_MAX', the current setting is not changed.
+   is compressed with Deflate encoding, `HDBTBZIP' specifies that each record is compressed with
+   BZIP2 encoding, `HDBTTCBS' specifies that each record is compressed with TCBS encoding.  If it
+   is `UINT8_MAX', the current setting is not changed.
    If successful, the return value is true, else, it is false.
    This function is useful to reduce the size of the database file with data fragmentation by
    successive updating. */
@@ -562,13 +549,19 @@ void tchdbsetdbgfd(TCHDB *hdb, int fd);
 int tchdbdbgfd(TCHDB *hdb);
 
 
-/* Synchronize updating contents on memory.
+/* Synchronize updating contents on memory of a hash database object.
    `hdb' specifies the hash database object connected as a writer.
    `phys' specifies whether to synchronize physically.
    If successful, the return value is true, else, it is false. */
 bool tchdbmemsync(TCHDB *hdb, bool phys);
 
 
+/* Clear the cache of a hash tree database object.
+   `bdb' specifies the hash tree database object.
+   If successful, the return value is true, else, it is false. */
+bool tchdbcacheclear(TCHDB *hdb);
+
+
 /* Get the number of elements of the bucket array of a hash database object.
    `hdb' specifies the hash database object.
    The return value is the number of elements of the bucket array or 0 if the object does not
@@ -641,6 +634,59 @@ char *tchdbopaque(TCHDB *hdb);
 uint64_t tchdbbnumused(TCHDB *hdb);
 
 
+/* Set the custom codec functions of a hash database object.
+   `hdb' specifies the hash database object.
+   `enc' specifies the pointer to the custom encoding function.
+   `encop' specifies an arbitrary pointer to be given as a parameter of the encoding function.
+   If it is not needed, `NULL' can be specified.
+   `dec' specifies the pointer to the custom decoding function.
+   `decop' specifies an arbitrary pointer to be given as a parameter of the decoding function.
+   If it is not needed, `NULL' can be specified.
+   If successful, the return value is true, else, it is false.
+   Note that the custom codec functions should be set before the database is opened and should be
+   set every time the database is being opened. */
+bool tchdbsetcodecfunc(TCHDB *hdb, HDBCODEC enc, void *encop, HDBCODEC dec, void *decop);
+
+
+/* Get the custom codec functions of a hash database object.
+   `hdb' specifies the hash database object.
+   `ep' specifies the pointer to a variable into which the pointer to the custom encoding
+   function is assigned
+   `eop' specifies the pointer to a variable into which the arbitrary pointer to be given to the
+   encoding function is assigned.
+   `dp' specifies the pointer to a variable into which the pointer to the custom decoding
+   function is assigned
+   `dop' specifies the pointer to a variable into which the arbitrary pointer to be given to the
+   decoding function is assigned. */
+void tchdbcodecfunc(TCHDB *hdb, HDBCODEC *ep, void **eop, HDBCODEC *dp, void **dop);
+
+
+/* Retrieve the next record of a record in a hash database object.
+   `hdb' specifies the hash database object.
+   `kbuf' specifies the pointer to the region of the key.  If it is `NULL', the first record is
+   retrieved.
+   `ksiz' specifies the size of the region of the key.
+   `sp' specifies the pointer to the variable into which the size of the region of the return
+   value is assigned.
+   If successful, the return value is the pointer to the region of the value of the next record.
+   `NULL' is returned if no record corresponds.
+   Because an additional zero code is appended at the end of the region of the return value,
+   the return value can be treated as a character string.  Because the region of the return
+   value is allocated with the `malloc' call, it should be released with the `free' call when
+   it is no longer in use. */
+void *tchdbgetnext(TCHDB *hdb, const void *kbuf, int ksiz, int *sp);
+
+
+/* Retrieve the next record of a string record in a hash database object.
+   `hdb' specifies the hash database object.
+   `kstr' specifies the string of the key.  If it is `NULL', the first record is retrieved.
+   If successful, the return value is the string of the value of the next record.  `NULL' is
+   returned if no record corresponds.
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call when it is no longer in use. */
+char *tchdbgetnext2(TCHDB *hdb, const char *kstr);
+
+
 
 __TCHDB_CLINKAGEEND
 #endif                                   /* duplication check */
