@@ -90,14 +90,14 @@ static void usage(void){
   fprintf(stderr, "%s: the command line utility of the hash database API\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s create [-tl] [-td|-tb] path [bnum [apow [fpow]]]\n", g_progname);
+  fprintf(stderr, "  %s create [-tl] [-td|-tb|-tt|-tx] path [bnum [apow [fpow]]]\n", g_progname);
   fprintf(stderr, "  %s inform [-nl|-nb] path\n", g_progname);
   fprintf(stderr, "  %s put [-nl|-nb] [-sx] [-dk|-dc] path key value\n", g_progname);
   fprintf(stderr, "  %s out [-nl|-nb] [-sx] path key\n", g_progname);
   fprintf(stderr, "  %s get [-nl|-nb] [-sx] [-px] [-pz] path key\n", g_progname);
   fprintf(stderr, "  %s list [-nl|-nb] [-m num] [-pv] [-px] [-fm str] path\n", g_progname);
-  fprintf(stderr, "  %s optimize [-tl] [-td|-tb] [-tz] [-nl|-nb] path [bnum [apow [fpow]]]\n",
-          g_progname);
+  fprintf(stderr, "  %s optimize [-tl] [-td|-tb|-tt|-tx] [-tz] [-nl|-nb] path"
+          " [bnum [apow [fpow]]]\n", g_progname);
   fprintf(stderr, "  %s importtsv [-nl|-nb] [-sc] path [file]\n", g_progname);
   fprintf(stderr, "  %s version\n", g_progname);
   fprintf(stderr, "\n");
@@ -153,19 +153,26 @@ static char *hextoobj(const char *str, int *sp){
 
 /* read a line from a file descriptor */
 static char *mygetline(FILE *ifp){
-  char *buf;
-  int c, len, blen;
-  buf = NULL;
-  len = 0;
-  blen = 256;
+  int len = 0;
+  int blen = 1024;
+  char *buf = tcmalloc(blen);
+  bool end = true;
+  int c;
   while((c = fgetc(ifp)) != EOF){
-    if(blen <= len) blen *= 2;
-    buf = tcrealloc(buf, blen + 1);
+    end = false;
+    if(c == '\0') continue;
+    if(blen <= len){
+      blen *= 2;
+      buf = tcrealloc(buf, blen + 1);
+    }
     if(c == '\n' || c == '\r') c = '\0';
     buf[len++] = c;
     if(c == '\0') break;
   }
-  if(!buf) return NULL;
+  if(end){
+    tcfree(buf);
+    return NULL;
+  }
   buf[len] = '\0';
   return buf;
 }
@@ -185,7 +192,11 @@ static int runcreate(int argc, char **argv){
       } else if(!strcmp(argv[i], "-td")){
         opts |= HDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        opts |= HDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         opts |= HDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= HDBTEXCODEC;
       } else {
         usage();
       }
@@ -281,8 +292,8 @@ static int runput(int argc, char **argv){
     vbuf = tcmemdup(value, vsiz);
   }
   int rv = procput(path, kbuf, ksiz, vbuf, vsiz, omode, dmode);
-  free(vbuf);
-  free(kbuf);
+  tcfree(vbuf);
+  tcfree(kbuf);
   return rv;
 }
 
@@ -322,7 +333,7 @@ static int runout(int argc, char **argv){
     kbuf = tcmemdup(key, ksiz);
   }
   int rv = procout(path, kbuf, ksiz, omode);
-  free(kbuf);
+  tcfree(kbuf);
   return rv;
 }
 
@@ -368,7 +379,7 @@ static int runget(int argc, char **argv){
     kbuf = tcmemdup(key, ksiz);
   }
   int rv = procget(path, kbuf, ksiz, omode, px, pz);
-  free(kbuf);
+  tcfree(kbuf);
   return rv;
 }
 
@@ -429,8 +440,14 @@ static int runoptimize(int argc, char **argv){
         if(opts == UINT8_MAX) opts = 0;
         opts |= HDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        if(opts == UINT8_MAX) opts = 0;
+        opts |= HDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         if(opts == UINT8_MAX) opts = 0;
         opts |= HDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        if(opts == UINT8_MAX) opts = 0;
+        opts |= HDBTEXCODEC;
       } else if(!strcmp(argv[i], "-tz")){
         if(opts == UINT8_MAX) opts = 0;
       } else if(!strcmp(argv[i], "-nl")){
@@ -503,6 +520,7 @@ static int runversion(int argc, char **argv){
 static int proccreate(const char *path, int bnum, int apow, int fpow, int opts){
   TCHDB *hdb = tchdbnew();
   if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(hdb);
   if(!tchdbtune(hdb, bnum, apow, fpow, opts)){
     printerr(hdb);
     tchdbdel(hdb);
@@ -527,6 +545,7 @@ static int proccreate(const char *path, int bnum, int apow, int fpow, int opts){
 static int procinform(const char *path, int omode){
   TCHDB *hdb = tchdbnew();
   if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL);
   if(!tchdbopen(hdb, path, HDBOREADER | omode)){
     printerr(hdb);
     tchdbdel(hdb);
@@ -538,8 +557,9 @@ static int procinform(const char *path, int omode){
   printf("path: %s\n", npath);
   const char *type = "(unknown)";
   switch(tchdbtype(hdb)){
-  case HDBTHASH: type = "hash"; break;
-  case HDBTBTREE: type = "btree"; break;
+  case TCDBTHASH: type = "hash"; break;
+  case TCDBTBTREE: type = "btree"; break;
+  case TCDBTFIXED: type = "fixed"; break;
   }
   printf("database type: %s\n", type);
   uint8_t flags = tchdbflags(hdb);
@@ -560,7 +580,9 @@ static int procinform(const char *path, int omode){
   printf("options:");
   if(opts & HDBTLARGE) printf(" large");
   if(opts & HDBTDEFLATE) printf(" deflate");
+  if(opts & HDBTBZIP) printf(" bzip");
   if(opts & HDBTTCBS) printf(" tcbs");
+  if(opts & HDBTEXCODEC) printf(" excodec");
   printf("\n");
   printf("record number: %llu\n", (unsigned long long)tchdbrnum(hdb));
   printf("file size: %llu\n", (unsigned long long)tchdbfsiz(hdb));
@@ -578,6 +600,7 @@ static int procput(const char *path, const char *kbuf, int ksiz, const char *vbu
                    int omode, int dmode){
   TCHDB *hdb = tchdbnew();
   if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(hdb);
   if(!tchdbopen(hdb, path, HDBOWRITER | omode)){
     printerr(hdb);
     tchdbdel(hdb);
@@ -617,6 +640,7 @@ static int procput(const char *path, const char *kbuf, int ksiz, const char *vbu
 static int procout(const char *path, const char *kbuf, int ksiz, int omode){
   TCHDB *hdb = tchdbnew();
   if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(hdb);
   if(!tchdbopen(hdb, path, HDBOWRITER | omode)){
     printerr(hdb);
     tchdbdel(hdb);
@@ -640,6 +664,7 @@ static int procout(const char *path, const char *kbuf, int ksiz, int omode){
 static int procget(const char *path, const char *kbuf, int ksiz, int omode, bool px, bool pz){
   TCHDB *hdb = tchdbnew();
   if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(hdb);
   if(!tchdbopen(hdb, path, HDBOREADER | omode)){
     printerr(hdb);
     tchdbdel(hdb);
@@ -651,7 +676,7 @@ static int procget(const char *path, const char *kbuf, int ksiz, int omode, bool
   if(vbuf){
     printdata(vbuf, vsiz, px);
     if(!pz) putchar('\n');
-    free(vbuf);
+    tcfree(vbuf);
   } else {
     printerr(hdb);
     err = true;
@@ -669,6 +694,7 @@ static int procget(const char *path, const char *kbuf, int ksiz, int omode, bool
 static int proclist(const char *path, int omode, int max, bool pv, bool px, const char *fmstr){
   TCHDB *hdb = tchdbnew();
   if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(hdb);
   if(!tchdbopen(hdb, path, HDBOREADER | omode)){
     printerr(hdb);
     tchdbdel(hdb);
@@ -687,7 +713,7 @@ static int proclist(const char *path, int omode, int max, bool pv, bool px, cons
         if(vbuf){
           putchar('\t');
           printdata(vbuf, vsiz, px);
-          free(vbuf);
+          tcfree(vbuf);
         }
       }
       putchar('\n');
@@ -726,6 +752,7 @@ static int proclist(const char *path, int omode, int max, bool pv, bool px, cons
 static int procoptimize(const char *path, int bnum, int apow, int fpow, int opts, int omode){
   TCHDB *hdb = tchdbnew();
   if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(hdb);
   if(!tchdbopen(hdb, path, HDBOWRITER | omode)){
     printerr(hdb);
     tchdbdel(hdb);
@@ -747,17 +774,18 @@ static int procoptimize(const char *path, int bnum, int apow, int fpow, int opts
 
 /* perform importtsv command */
 static int procimporttsv(const char *path, const char *file, int omode, bool sc){
-  TCHDB *hdb = tchdbnew();
-  if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
   FILE *ifp = file ? fopen(file, "rb") : stdin;
   if(!ifp){
     fprintf(stderr, "%s: could not open\n", file ? file : "(stdin)");
-    tchdbdel(hdb);
     return 1;
   }
+  TCHDB *hdb = tchdbnew();
+  if(g_dbgfd >= 0) tchdbsetdbgfd(hdb, g_dbgfd);
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)) printerr(hdb);
   if(!tchdbopen(hdb, path, HDBOWRITER | HDBOCREAT | omode)){
     printerr(hdb);
     tchdbdel(hdb);
+    if(ifp != stdin) fclose(ifp);
     return 1;
   }
   bool err = false;
@@ -765,14 +793,17 @@ static int procimporttsv(const char *path, const char *file, int omode, bool sc)
   int cnt = 0;
   while(!err && (line = mygetline(ifp)) != NULL){
     char *pv = strchr(line, '\t');
-    if(!pv) continue;
+    if(!pv){
+      tcfree(line);
+      continue;
+    }
     *pv = '\0';
     if(sc) tcstrtolower(line);
     if(!tchdbput2(hdb, line, pv + 1) && tchdbecode(hdb) != TCEKEEP){
       printerr(hdb);
       err = true;
     }
-    free(line);
+    tcfree(line);
     if(cnt > 0 && cnt % 100 == 0){
       putchar('.');
       fflush(stdout);
