@@ -123,13 +123,13 @@ static void usage(void){
   fprintf(stderr, "%s: test cases of the hash database API of Tokyo Cabinet\n", g_progname);
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
-  fprintf(stderr, "  %s write [-tl] [-td|-tb] [-nl|-nb] [-as] [-rnd] path tnum rnum"
+  fprintf(stderr, "  %s write [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-as] [-rnd] path tnum rnum"
           " [bnum [apow [fpow]]]\n", g_progname);
   fprintf(stderr, "  %s read [-rc num] [-nl|-nb] [-wb] [-rnd] path tnum\n", g_progname);
-  fprintf(stderr, "  %s remove [-nl|-nb] [-wb] [-rnd] path tnum\n", g_progname);
-  fprintf(stderr, "  %s wicked [-tl] [-td|-tb] [-nl|-nb] [-nc] path tnum rnum\n",
+  fprintf(stderr, "  %s remove [-nl|-nb] [-rnd] path tnum\n", g_progname);
+  fprintf(stderr, "  %s wicked [-tl] [-td|-tb|-tt|-tx] [-nl|-nb] [-nc] path tnum rnum\n",
           g_progname);
-  fprintf(stderr, "  %s typical [-tl] [-td|-tb] [-rc num] [-nl|-nb] [-nc] [-rr num]"
+  fprintf(stderr, "  %s typical [-tl] [-td|-tb|-tt|-tx] [-rc num] [-nl|-nb] [-nc] [-rr num]"
           " path tnum rnum [bnum [apow [fpow]]]\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
@@ -209,7 +209,11 @@ static int runwrite(int argc, char **argv){
       } else if(!strcmp(argv[i], "-td")){
         opts |= HDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        opts |= HDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         opts |= HDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= HDBTEXCODEC;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= HDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -337,7 +341,11 @@ static int runwicked(int argc, char **argv){
       } else if(!strcmp(argv[i], "-td")){
         opts |= HDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        opts |= HDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         opts |= HDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= HDBTEXCODEC;
       } else if(!strcmp(argv[i], "-nl")){
         omode |= HDBONOLCK;
       } else if(!strcmp(argv[i], "-nb")){
@@ -386,7 +394,11 @@ static int runtypical(int argc, char **argv){
       } else if(!strcmp(argv[i], "-td")){
         opts |= HDBTDEFLATE;
       } else if(!strcmp(argv[i], "-tb")){
+        opts |= HDBTBZIP;
+      } else if(!strcmp(argv[i], "-tt")){
         opts |= HDBTTCBS;
+      } else if(!strcmp(argv[i], "-tx")){
+        opts |= HDBTEXCODEC;
       } else if(!strcmp(argv[i], "-rc")){
         if(++i >= argc) usage();
         rcnum = atoi(argv[i]);
@@ -444,6 +456,10 @@ static int procwrite(const char *path, int tnum, int rnum, int bnum, int apow, i
     eprint(hdb, "tchdbsetmutex");
     err = true;
   }
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(hdb, "tchdbsetcodecfunc");
+    err = true;
+  }
   if(!tchdbtune(hdb, bnum, apow, fpow, opts)){
     eprint(hdb, "tchdbtune");
     err = true;
@@ -511,6 +527,10 @@ static int procread(const char *path, int tnum, int rcnum, int omode, bool wb, b
     eprint(hdb, "tchdbsetmutex");
     err = true;
   }
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(hdb, "tchdbsetcodecfunc");
+    err = true;
+  }
   if(!tchdbsetcache(hdb, rcnum)){
     eprint(hdb, "tchdbsetcache");
     err = true;
@@ -578,6 +598,10 @@ static int procremove(const char *path, int tnum, int omode, bool rnd){
     eprint(hdb, "tchdbsetmutex");
     err = true;
   }
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(hdb, "tchdbsetcodecfunc");
+    err = true;
+  }
   if(!tchdbopen(hdb, path, HDBOWRITER | omode)){
     eprint(hdb, "tchdbopen");
     err = true;
@@ -640,6 +664,10 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
     eprint(hdb, "tchdbsetmutex");
     err = true;
   }
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(hdb, "tchdbsetcodecfunc");
+    err = true;
+  }
   if(!tchdbtune(hdb, rnum / 50, 2, -1, opts)){
     eprint(hdb, "tchdbtune");
     err = true;
@@ -673,7 +701,6 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
       targs[i].nc = nc;
       targs[i].id = i;
       targs[i].map = map;
-      targs[i].map = map;
       if(pthread_create(threads + i, NULL, threadwicked, targs + i) != 0){
         eprint(hdb, "pthread_create");
         targs[i].id = -1;
@@ -724,7 +751,7 @@ static int procwicked(const char *path, int tnum, int rnum, int opts, int omode,
           err = true;
         }
       }
-      free(rbuf);
+      tcfree(rbuf);
       if(i % 50 == 0) iprintf(" (%08d)\n", i);
     }
     if(rnum % 50 > 0) iprintf(" (%08d)\n", rnum);
@@ -758,6 +785,10 @@ static int proctypical(const char *path, int tnum, int rnum, int bnum, int apow,
     eprint(hdb, "tchdbsetmutex");
     err = true;
   }
+  if(!tchdbsetcodecfunc(hdb, _tc_recencode, NULL, _tc_recdecode, NULL)){
+    eprint(hdb, "tchdbsetcodecfunc");
+    err = true;
+  }
   if(!tchdbtune(hdb, bnum, apow, fpow, opts)){
     eprint(hdb, "tchdbtune");
     err = true;
@@ -831,7 +862,7 @@ static void *threadwrite(void *targ){
     int len = sprintf(buf, "%08d", base + (rnd ? myrand(i) : i));
     if(as){
       if(!tchdbputasync(hdb, buf, len, buf, len)){
-        eprint(hdb, "tchdbput");
+        eprint(hdb, "tchdbputasync");
         err = true;
         break;
       }
@@ -878,7 +909,7 @@ static void *threadread(void *targ){
         eprint(hdb, "tchdbget");
         err = true;
       }
-      free(vbuf);
+      tcfree(vbuf);
     }
     if(id == 0 && rnum > 250 && i % (rnum / 250) == 0){
       putchar('.');
@@ -1053,7 +1084,7 @@ static void *threadwicked(void *targ){
         err = true;
       }
       if(!nc) tcmapput(map, kbuf, ksiz, rbuf, vsiz);
-      free(rbuf);
+      tcfree(rbuf);
       break;
     case 11:
       if(id == 0) putchar('B');
@@ -1061,15 +1092,15 @@ static void *threadwicked(void *targ){
         eprint(hdb, "tchdbget");
         err = true;
       }
-      free(rbuf);
+      tcfree(rbuf);
       break;
     case 12:
       if(id == 0) putchar('C');
       if(!(rbuf = tchdbget2(hdb, kbuf)) && tchdbecode(hdb) != TCENOREC){
-        eprint(hdb, "tchdbget");
+        eprint(hdb, "tchdbget2");
         err = true;
       }
-      free(rbuf);
+      tcfree(rbuf);
       break;
     case 13:
       if(id == 0) putchar('D');
@@ -1102,7 +1133,7 @@ static void *threadwicked(void *targ){
           int iksiz;
           char *ikbuf = tchdbiternext(hdb, &iksiz);
           if(ikbuf){
-            free(ikbuf);
+            tcfree(ikbuf);
           } else {
             int ecode = tchdbecode(hdb);
             if(ecode != TCEINVALID && ecode != TCENOREC){
@@ -1200,7 +1231,7 @@ static void *threadtypical(void *targ){
         int ksiz;
         char *kbuf = tchdbiternext(hdb, &ksiz);
         if(kbuf){
-          free(kbuf);
+          tcfree(kbuf);
         } else if(tchdbecode(hdb) != TCEINVALID && tchdbecode(hdb) != TCENOREC){
           eprint(hdb, "tchdbiternext");
           err = true;
@@ -1213,12 +1244,12 @@ static void *threadtypical(void *targ){
         if(map){
           int msiz;
           const char *mbuf = tcmapget(map, buf, len, &msiz);
-          if(msiz != vsiz || memcmp(mbuf, vbuf, vsiz)){
+          if(!mbuf || msiz != vsiz || memcmp(mbuf, vbuf, vsiz)){
             eprint(hdb, "(validation)");
             err = true;
           }
         }
-        free(vbuf);
+        tcfree(vbuf);
       } else {
         if(tchdbecode(hdb) != TCENOREC){
           eprint(hdb, "tchdbget");
@@ -1250,7 +1281,7 @@ static void *threadtypical(void *targ){
           eprint(hdb, "(validation)");
           err = true;
         }
-        free(vbuf);
+        tcfree(vbuf);
       } else {
         eprint(hdb, "(validation)");
         err = true;
